import { registerExtension } from 'vscode/extensions';

const manifest = {"name":"language-haskell","displayName":"Haskell Syntax Highlighting","description":"Syntax support for the Haskell programming language.","version":"3.6.0","publisher":"justusadam","engines":{"vscode":"^1.0.0"},"icon":"images/logo.png","galleryBanner":{"color":"#293845","theme":"dark"},"files":["src/*","syntaxes/*.json","images/","README.md","LICENSE","*-configuration.json","tsconfig.json","CHANGELOG.md"],"keywords":["language","haskell","highlight","syntax"],"license":"BSD-3-Clause","homepage":"https://github.com/JustusAdam/language-haskell","repository":{"type":"git","url":"https://github.com/JustusAdam/language-haskell.git"},"bugs":{"url":"https://github.com/JustusAdam/language-haskell/issues","email":"dev@justus.science"},"categories":["Programming Languages"],"contributes":{"configuration":{"type":"object","title":"Haskell configuration","properties":{"haskell.indentationRules.enabled":{"type":"boolean","default":true,"description":"Set to 'false' to disable automatically increasing indent on newlines after '=', 'do', 'where' etc."}}},"languages":[{"id":"haskell","aliases":["Haskell","haskell"],"extensions":[".hs",".hsig","hs-boot"],"configuration":"./haskell-configuration.json"},{"id":"cabal","aliases":["Cabal","cabal"],"extensions":[".cabal"],"configuration":"./cabal-configuration.json"},{"id":"C2Hs","aliases":["C2Hs","c2hs"],"extensions":[".chs"],"configuration":"./C2Hs-configuration.json"},{"id":"Hsc2Hs","aliases":["Hsc2Hs","HsC2Hs","hsc2hs"],"extensions":[".hsc"],"configuration":"./Hsc2Hs-configuration.json"},{"id":"literate haskell","aliases":["Literate Haskell","literate Haskell"],"extensions":[".lhs"],"configuration":"./literate-haskell-configuration.json"},{"id":"yaml","aliases":["YAML","yaml"],"filenames":["stack.yaml.lock"]},{"id":"alex","aliases":["Alex","alex","Haskell Alex"],"extensions":[".x"],"configuration":"./haskell-configuration.json"},{"id":"happy","aliases":["Happy","happy","Haskell Happy"],"extensions":[".y"],"configuration":"./haskell-configuration.json"}],"grammars":[{"language":"haskell","scopeName":"source.haskell","path":"./syntaxes/haskell.json"},{"language":"cabal","scopeName":"source.cabal","path":"./syntaxes/cabal.json"},{"language":"C2Hs","scopeName":"source.c2hs","path":"./syntaxes/C2Hs.json"},{"language":"Hsc2Hs","scopeName":"source.hsc","path":"./syntaxes/Hsc2Hs.json"},{"language":"literate haskell","scopeName":"text.tex.latex.haskell","path":"./syntaxes/literateHaskell.json"},{"language":"alex","scopeName":"source.haskell.alex","path":"./syntaxes/alex.json"},{"language":"happy","scopeName":"source.haskell.happy","path":"./syntaxes/happy.json"},{"scopeName":"markdown.haskell.codeblock","path":"./syntaxes/codeblock-haskell.json","injectTo":["text.html.markdown"],"embeddedLanguages":{"meta.embedded.block.haskell":"haskell"}},{"scopeName":"markdown.lhaskell.codeblock","path":"./syntaxes/codeblock-literate-haskell.json","injectTo":["text.html.markdown"],"embeddedLanguages":{"meta.embedded.block.lhaskell":"lhaskell"}},{"scopeName":"markdown.cabal.codeblock","path":"./syntaxes/codeblock-cabal.json","injectTo":["text.html.markdown"],"embeddedLanguages":{"meta.embedded.block.cabal":"cabal"}}]}};

const { registerFileUrl, whenReady } = registerExtension(manifest, undefined, {"system":true,"readmePath":"README.md","changelogPath":"CHANGELOG.md"});


registerFileUrl('README.md', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/README.md', import.meta.url).href, {"mimeType":"text/markdown","size":3072});

registerFileUrl('CHANGELOG.md', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/CHANGELOG.md', import.meta.url).href, {"mimeType":"text/markdown","size":19110});

registerFileUrl('syntaxes/haskell.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/haskell.json', import.meta.url).href, {"mimeType":"application/json","size":87624});

registerFileUrl('syntaxes/cabal.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/cabal.json', import.meta.url).href, {"mimeType":"application/json","size":2457});

registerFileUrl('syntaxes/C2Hs.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/C2Hs.json', import.meta.url).href, {"mimeType":"application/json","size":445});

registerFileUrl('syntaxes/Hsc2Hs.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/Hsc2Hs.json', import.meta.url).href, {"mimeType":"application/json","size":487});

registerFileUrl('syntaxes/literateHaskell.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/literateHaskell.json', import.meta.url).href, {"mimeType":"application/json","size":1375});

registerFileUrl('syntaxes/alex.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/alex.json', import.meta.url).href, {"mimeType":"application/json","size":5594});

registerFileUrl('syntaxes/happy.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/happy.json', import.meta.url).href, {"mimeType":"application/json","size":4410});

registerFileUrl('syntaxes/codeblock-haskell.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/codeblock-haskell.json', import.meta.url).href, {"mimeType":"application/json","size":821});

registerFileUrl('syntaxes/codeblock-literate-haskell.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/codeblock-literate-haskell.json', import.meta.url).href, {"mimeType":"application/json","size":852});

registerFileUrl('syntaxes/codeblock-cabal.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/codeblock-cabal.json', import.meta.url).href, {"mimeType":"application/json","size":806});

registerFileUrl('haskell-configuration.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/haskell-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":640});

registerFileUrl('cabal-configuration.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/cabal-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":73});

registerFileUrl('C2Hs-configuration.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/C2Hs-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":586});

registerFileUrl('Hsc2Hs-configuration.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/Hsc2Hs-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":644});

registerFileUrl('literate-haskell-configuration.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/literate-haskell-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":639});

registerFileUrl('package.json', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/package.json', import.meta.url).href, {"mimeType":"application/json","size":5363});

registerFileUrl('images/logo.png', new URL('../assets/justusadam.language-haskell-3.6.0.vsix/logo.png', import.meta.url).href, {"mimeType":"image/png","size":4653});

export { whenReady };
